function getTimeString(date) {
    return date.getHours() + ":" + date.getMinutes() + ":" + date.getSeconds();
}

function getDateString(date) {
    return date.getFullYear() + "-" + (date.getMonth() + 1) + "-" + date.getDate();
}

function getMessageHeaderSignValue(header) {
    if (header.lastIndexOf('<') === -1 && header.lastIndexOf('>') === -1) {
        return header;
    } else {
        return header.slice(header.lastIndexOf('<') + 1, header.lastIndexOf('>'));
    }
}

function bigRandomNumber() {
    return Math.floor(Math.random() * 1234567890123);
}

function findContent(contentType, msgParts) {
    let value = '';
    if (msgParts != null && msgParts.length > 0) {
        for (let i = 0; i < msgParts.length; i++) {
            if (msgParts[i].contentType === contentType) {
                return msgParts[i].body;
            } else if (msgParts[i].parts != null) {
                value = findContent(contentType, msgParts[i].parts);
                if (value !== '') {
                    return value;
                }
            }
        }
    }
    return value;
}

function findRecipients(message) {
    let recipients = [];

    message.recipients.forEach(recipient => {
		if(recipient.includes("@")) {
			recipients.push(getMessageHeaderSignValue(recipient));
		}
    });

    message.bccList.forEach(recipient => {
		if(recipient.includes("@")) {
			recipients.push(getMessageHeaderSignValue(recipient));
		}
    });

    message.ccList.forEach(recipient => {
		if(recipient.includes("@")) {
			recipients.push(getMessageHeaderSignValue(recipient));
		}
    });
    return recipients;
}

function getAttachmentForImport(attachmentFile) {

    let attachmentToImport = {
        name: attachmentFile.leafName,
        content: ""
    }
    let fileInputStream, binaryInputStream;
    try {
        fileInputStream = Components.classes["@mozilla.org/network/file-input-stream;1"].createInstance(Components.interfaces.nsIFileInputStream);
        fileInputStream.init(attachmentFile, 0x01, 700, null);

        binaryInputStream = Components.classes["@mozilla.org/binaryinputstream;1"].createInstance(Components.interfaces.nsIBinaryInputStream);
        binaryInputStream.setInputStream(fileInputStream);

        attachmentToImport.content = window.btoa(binaryInputStream.readBytes(attachmentFile.fileSizeOfLink));

        binaryInputStream.close();
        fileInputStream.close();
    } catch(e) {

    }
    return attachmentToImport;
}

function saveAttachmentToTemp(attachment) {
    let folderDelimiter = '/';
    if (window.navigator.platform.toLowerCase().indexOf("win") !== -1) {
        folderDelimiter = '\\';
    }
    let tempDirectory = Components.classes["@mozilla.org/file/directory_service;1"].getService(Components.interfaces.nsIProperties).get("TmpD", Components.interfaces.nsIFile).path;
    let filePath = tempDirectory + folderDelimiter + 'crm_import';

    let fileName = window.encodeURIComponent(attachment.displayName);

    let destinationFolder = Components.classes["@mozilla.org/file/local;1"].createInstance(Components.interfaces.nsIFile);
    destinationFolder.initWithPath(filePath);

    let nativeMessenger = Components.classes['@mozilla.org/messenger;1'].getService(Components.interfaces.nsIMessenger);

    let file = nativeMessenger.saveAttachmentToFolder(attachment.contentType, attachment.url, fileName, attachment.uri, destinationFolder);

    if (!file) {
        Services.prompt.alert(window, "CRM Vision", "Wystąpił błąd przy dodawaniu pliku (załącznika) do systemu CRM. Sprawdź czy masz uprawnienia zapisu do folderu " + filePath + " oraz czy masz wystarczającą ilość miejsca na dysku.");
    }

    return file;
}

function getUserAuth() {
    let result = {};

    result['username'] = Services.prefs.getStringPref("extensions.crm_vision_extension.settings.username");
    result['password'] = Services.prefs.getStringPref("extensions.crm_vision_extension.settings.password");
    result['company_name'] = Services.prefs.getStringPref("extensions.crm_vision_extension.settings.companyname");

    return result;
}

function loadAttachments() {

    let nativeMessenger = Components.classes['@mozilla.org/messenger;1'].getService(Components.interfaces.nsIMessenger);
    let selectedMessageUri = Components.classes["@mozilla.org/appshell/window-mediator;1"].getService(Components.interfaces.nsIWindowMediator).getEnumerator("mail:3pane").getNext().gMessageDisplay.folderDisplay.selectedMessageUris[0];

    let messengerService = nativeMessenger.messageServiceFromURI(selectedMessageUri).QueryInterface(Components.interfaces.nsIMsgMessageService);
    let streamWorker = AttachmentWorker();
    streamWorker.attachments = [];
    streamWorker.msgUri = selectedMessageUri;
    messengerService.streamMessage(selectedMessageUri, streamWorker, null, null, true, "filter");
}

function importAttachment(loader,attachmentFile,previousAttachment) {
    if (attachmentFile) {
        window.setTimeout(function() {
            let currentAttachment = getAttachmentForImport(attachmentFile);
            if (previousAttachment && (currentAttachment.content.length !== 0 && previousAttachment.content.length === currentAttachment.content.length)) {
                crmVisionSoapApi.importAttachment(messageToImport,currentAttachment);
                attachmentFile.remove(true);
                loader.postMessage("uploadFinished");
            } else {
                importAttachment(loader,attachmentFile,currentAttachment);
            }
        },1000);
    }
}