async function getServerUrlDataHandler(data) {
    messageToImport.globalServerUrl = SOAPTools.prepareResultFetch(data).value;

    if (messageToImport.globalServerUrl) {
        let selectedMessages = notifyTools.notifyBackground({command: "getSelectedMessages"});
        
        selectedMessages.then(value => {
            let message = value.messages[0];

            if (message === undefined) {
                Services.prompt.alert(window, "CRM Vision", "Nie wybrano wiadomości");
            } else {
                thunderbirdMessageId = message.id;

                let freshMessage = notifyTools.notifyBackground({command: "getMessage", idMessage: message.id});
                freshMessage.then(message => {

                    if (messageToImport.message.isInbound) {
                        messageToImport.message.emails = getMessageHeaderSignValue(message.author);
                    } else {
                        messageToImport.message.emails = findRecipients(message).join(',');
                    }

                    messageToImport.message.emailDate = getDateString(message.date) + " " + getTimeString(message.date);
                    messageToImport.message.subject = message.subject;

                    let fullMessage = notifyTools.notifyBackground({command: "getFullMessage", idMessage: message.id});
                    fullMessage.then(fullMessage => {
                        messageToImport.message.outerId = getMessageHeaderSignValue(fullMessage.headers['message-id'][0]) + getDateString(message.date) + getTimeString(message.date) + getMessageHeaderSignValue(message.author);

                        crmVisionSoapApi.checkIfEmailExists(messageToImport).then(checkIfEmailExistsHandler).then(function () {
                            messageToImport.message.emailBodyPlain = findContent('text/plain', fullMessage.parts);
                            messageToImport.message.emailBodyHTML = findContent('text/html', fullMessage.parts);

                            if (messageToImport.message.emailBodyHTML === '') {
                                messageToImport.message.emailBodyHTML = messageToImport.message.emailBodyPlain;
                            }

                            let emailsToGetIdContacts = messageToImport.message.isInbound ? [messageToImport.message.emails] : messageToImport.message.emails.split(',');
                            emailsToGetIdContacts.forEach(email => {
                                contactsPromises.push(crmVisionSoapApi.getContacts(messageToImport, email, getContactsDataHandler));
                            });
                            Promise.all(contactsPromises).then(() => {
                                let numberOfContactsAutomaticallyChoosed = messageToImport.message.idContacts.length;
                                let interval = window.setInterval(function () {
                                    let numberOfClosedWindows = contactWindows.filter(w => w.closed).length;
                                    if (contactWindows.length === 0 || numberOfClosedWindows === contactWindows.length) {
                                        window.clearInterval(interval);
                                        if ((numberOfContactsAutomaticallyChoosed + contactWindows.length) === messageToImport.message.idContacts.length) {
                                            processProjects();
                                        }
                                    }
                                }, 100);
                            });
                        }).catch(function (data) {
                            Services.prompt.alert(window, "CRM Vision", data);
                        });
                    });
                });
            }
        });
    } else {
        Services.prompt.alert(window, "CRM Vision", `Wystąpił błąd pobrania adresu. Sprawdź identyfikator firmy.`);
    }
}

function getProjectsByIdContactsHandler(data) {
    let result = SOAPTools.prepareResultFetch(data);
    let resultStatus = result.value.substring(0, 2);

    if (resultStatus === '00') {
        let projects = result.value
            .substring(2)
            .slice(0, -3)
            .split('#|#')
            .filter(entry => entry !== "")
            .map(e => {
                const entry = e.split('#@#');
                return {
                    idProject: entry[0],
                    name: entry[1]
                }
            });

        if (projects.length > 0) {
            let projectsWindow = window.open('chrome://crm_vision_extension/content/views/projects.xhtml', 'projectsWindow',
                'chrome,toolbar=yes,titlebar,centerscreen,directories=no,status=yes,menubar=yes,scrollbars=no,resizable=yes,copyhistory=no,height=420,width=400'
            );
            projectsWindow.addEventListener("DOMContentLoaded", function () {
                projectsWindow.postMessage(projects);
            })
            projectWindows.push(projectsWindow);
        }
    }
}

function checkIfEmailExistsHandler(data) {
    let result = data !== undefined ? SOAPTools.prepareResultFetch(data) : "";

    if (result === "") {
        throw 'Posiadasz nieaktualną wersję systemu CRM Vision. Skontaktuj się z nami - tel: 58 690-41-50, 515-229-793, email: biuro@crmvision.pl w celu dokonania aktualizacji.';
    }

    let resultStatus = result.value.substring(0, 2);

    if (resultStatus !== '00') {
        throw result.value.substring(2);
    }
}

function getImportEmailDataHandler(data) {
    let importResult = SOAPTools.prepareResultFetch(data).value;
    let statusCode = importResult.substring(0, 2);
    let message = importResult.substring(2);

    if (statusCode === '00') {
        let check = {value: false};
        Services.prompt.confirmEx(null, "CRM Vision", message, Services.prompt.BUTTON_TITLE_OK, "OK", null, null, null, check);

        processImportAttachments();
    } else {
        Services.prompt.alert(window, "CRM Vision", message);
        if (statusCode === '01') {
            processImportAttachments();
        }
    }
}

function getContactsDataHandler(data) {
    let contacts = SOAPTools.prepareResultFetch(data).value
        .substring(2)
        .split('#|#')
        .map(e => {
            const entry = e.split('#@#');
            return {
                idContact: entry[0],
                shortName: entry[1]
            }
        })
        .filter(e => e.idContact !== "");

    if (contacts.length > 0) {
        if (contacts.length === 1) {
            messageToImport.message.idContacts.push(contacts[0].idContact);
        } else {
            let interval = window.setInterval(function () {
                if (contactWindows.filter(w => !w.closed).length === 0) {
                    let contactsWindow = window.open('chrome://crm_vision_extension/content/views/contacts.xhtml', 'contactsWindow' + bigRandomNumber(),
                        'chrome,toolbar=yes,titlebar,centerscreen,directories=no,status=yes,menubar=yes,scrollbars=no,copyhistory=no,resizable=yes,height=370,width=400'
                    );
                    contactsWindow.addEventListener("DOMContentLoaded", function () {
                        contactsWindow.postMessage({
                            contacts: contacts
                        });
                    });
                    contactWindows.push(contactsWindow);
                    window.clearInterval(interval);
                }
            }, 100);

        }
    }
}