soapResult = {
    fault: false,
    error: {
        number: 0,
        message: null,
    },
    value: null
};

SOAPTools = {
    buildRequest: function (nsURI, soapMethod, params) {
        let requestRoot = document.implementation.createDocument("http://schemas.xmlsoap.org/soap/envelope/", "Envelope");
        let requestEnvelope = requestRoot.getElementsByTagName("Envelope")[0];
        let requestBody = requestRoot.createElementNS("http://schemas.xmlsoap.org/soap/envelope/", "Body");
        let requestSOAPMethod = requestRoot.createElementNS(nsURI, soapMethod);

        requestEnvelope.appendChild(requestBody);
        requestBody.appendChild(requestSOAPMethod);

        for (const key in params) {
            let elem = requestRoot.createElementNS("http://www.w3.org/2001/XMLSchema-instance", key);
            elem.appendChild(requestRoot.createTextNode(params[key]));

            requestSOAPMethod.appendChild(elem);
        }

        return requestRoot;
    },
    executeRequestFetch: function (url, data, soapMethod, dataHandler, errorHandler) {
        let headers = new window.Headers({
            'Content-Type': 'text/xml'
        });
        let initObject = {
            method: 'POST',
            headers: headers,
            body: new window.XMLSerializer().serializeToString(data.documentElement)
        }

        let prepareResponse = function (response) {
            if (response.ok) {
                return response.text();
            } else {
                soapResult.fault = true;
                soapResult.error.number = response.status;
                soapResult.error.message = response.statusText;
                throw soapResult;
            }
        }

        let defaultErrorHandler = function (error) {
            let companyName = Services.prefs.getStringPref("extensions.crm_vision_extension.settings.companyname");
            let login = Services.prefs.getStringPref("extensions.crm_vision_extension.settings.username");
            let password = Services.prefs.getStringPref("extensions.crm_vision_extension.settings.password");

            if (companyName.length === 0 || login.length === 0 || password.length === 0) {
                Services.prompt.alert(window, "CRM Vision", `Najpierw skonfiguruj dodatek (Narzędzia -> CRM Vision -> Konfiguracja).`);
            } else {
                if (error.error) {
                    Services.prompt.alert(window, "CRM Vision", `Wystąpił błąd połączenia z serwerem. Numer błędu: ${error.error.number}, treść: ${error.error.message}`);
                } else {
                    Services.prompt.alert(window, "CRM Vision", `Wystąpił błąd połączenia z serwerem. Sprawdź konfigurację dodatku oraz swoje połączenie internetowe. Treść błędu: ${error}`);
                }
            }
        }

        return window.fetch(url, initObject)
            .then(prepareResponse)
            .then(typeof dataHandler === 'function' ? dataHandler : null)
            .catch(typeof errorHandler === 'function' ? errorHandler : defaultErrorHandler);
    },
    prepareResultFetch: function (responseData) {
        let parser = new window.DOMParser;
        let xmlDoc = parser.parseFromString(responseData, "text/xml");
        let result = xmlDoc.getElementsByTagName("return")[0];

        if (result.firstChild != null) {
            soapResult.fault = false;
            soapResult.value = result.firstChild.nodeValue;
        }
        return soapResult;
    },
    executeRequestXmlHttpRequest: function (xml, url) {
        let xmlHttpRequest = new window.XMLHttpRequest;
        xmlHttpRequest.open('POST', url, false);
        xmlHttpRequest.setRequestHeader('Content-Type', 'text/xml');
        xmlHttpRequest.send(xml);

        return xmlHttpRequest;
    },
    prepareResultXmlHttpResult: function (requestResult, soapMethod) {
        if (requestResult.readyState == 4 && requestResult.status == 200) {

            let generalErrorMessage = `Błąd połączenia z serwerem. Numer błędu: ${soapResult.error.number}, treść: ${soapResult.error.message}`;

            if (soapMethod === `get_server_url`) {
                generalErrorMessage = `Wystąpił błąd pobrania adresu. Sprawdź identyfikator firmy. Numer błędu: ${soapResult.error.number}, treść: ${soapResult.error.message}`;
            }

            let parser = new window.DOMParser;
            let xmlDoc = parser.parseFromString(requestResult.responseText, "text/xml");
            let result = xmlDoc.getElementsByTagName("return")[0];

            if (result.firstChild != null) {
                soapResult.fault = false;
                soapResult.value = result.firstChild.nodeValue;
            } else {
                soapResult.fault = true;
                soapResult.value = generalErrorMessage;
            }
        } else {

            let generalErrorMessage = `Błąd połączenia z serwerem. Numer błędu: ${soapResult.error.number}, treść: ${soapResult.error.message}`;

            if (soapMethod === `get_server_url`) {
                generalErrorMessage = `Wystąpił błąd pobrania adresu. Sprawdź identyfikator firmy. Numer błędu: ${soapResult.error.number}, treść: ${soapResult.error.message}`;
            }

            soapResult.fault = true;
            soapResult.error.number = requestResult.name;
            soapResult.error.message = requestResult.message;
            soapResult.value = generalErrorMessage;
        }

        return soapResult;
    }
}